/*
 * Copyright 2020 - All rights reserved.
 * Rodrigo Tapia-McClung
 *
 * November 2020
 */

/* global mapboxgl, turf */

const baseUrl = new URL(`/data`, window.location.href).href;
let queryGeom = `${baseUrl}/query/dim_municipio?columns=st_asgeojson(box2d(st_geometryn(municipio_geom_4326,1))) box,st_asgeojson(st_geometryn(municipio_geom_4326,1)) geom&filter=municipio_cvegeo='09002'`;
let tiles, polygon, box;

// define MVT layer for given table and some attributes
const mapboxLayer = (table, polygon, box) => {

    let columns = ["denue_nombre", "sector", "sector_nombre"],
        filter = `ST_Intersects(denue_geom_4326,ST_GeomFromGeoJSON('${JSON.stringify(box)}'))`;

    let pbfLayer = {
        id: table,
        source: {
            type: "vector",
            tiles: [`${baseUrl}/${table}/mvt/{z}/{x}/{y}?geom_column=denue_geom_4326&columns=${columns.join(",")}&filter=${filter}`],
            maxzoom: 19,
            minzoom: 6
        },
        "source-layer": table,
        type: "circle",
        "paint": {
            //"circle-opacity": 0.5,
            'circle-opacity': ['case', ['within', polygon], 0.5, 0],
            "circle-radius": 3.5,
            "circle-color": [
                "interpolate",
                ["linear"],
                ["get", "sector"],
                10, "rgb(49,54,149)",
                50, "rgb(255,255,191)",
                100, "rgb(158,1,66)"
            ]
        }
    }
    return pbfLayer;
}

let map = new mapboxgl.Map({
    container: 'mexmap',
    accessToken: "pk.eyJ1IjoiZGV2ZWxvcGdlbyIsImEiOiJja2dwcXFic20wYnJnMzBrbG11d3dwYTkyIn0.4WwFOH6C7hDQXV9obU6mAw",
    style: 'mapbox://styles/mapbox/dark-v10',
    center: [-99.18, 19.48],
    zoom: 12
});

map.addControl(new mapboxgl.NavigationControl());

map.on("style.load", async () => {
    d3.json(queryGeom).then( d => {
        polygon = JSON.parse(d[0].geom);
        box = JSON.parse(d[0].box);
        tiles = mapboxLayer("dim_denue2019", polygon, box);
        map.addLayer(tiles);
    });
});

map.on('click', e => {
    if (turf.booleanWithin(turf.point([e.lngLat.lng, e.lngLat.lat]), polygon)) {
        var features = map.queryRenderedFeatures(e.point,
            { layers: ['dim_denue2019'] }
        );

        if (features.length != 0) {
            new mapboxgl.Popup()
                .setLngLat(e.lngLat)
                .setHTML(features[0].properties['denue_nombre'])
                .addTo(map);
        }
    }
});

// Change the cursor to a pointer when the mouse is over the denue layer.
map.on('mouseenter', 'dim_denue2019', e => {
    if (turf.booleanWithin(turf.point([e.lngLat.lng, e.lngLat.lat]), polygon)) {
        map.getCanvas().style.cursor = 'pointer';
    }
});

// Change it back to a pointer when it leaves.
map.on('mouseleave', 'dim_denue2019', () => {
    map.getCanvas().style.cursor = '';
});